import {Injectable, OnDestroy} from '@angular/core'
import {Subject} from 'rxjs'
import {take, takeUntil} from 'rxjs/operators'
import {AppState} from '../../shared/app-state'
import {HttpClient, HttpResponse} from '@angular/common/http'
import {environment} from '../../../../environments/environment'
import {ApiService} from '../../shared/service/api.service'
import {User, Permission} from '../../shared/model/user'
import {ErrorService} from '../../shared/service/error.service'
import {globalLoadingWheel} from '../../shared/components/loading-wheel/loading-wheel.component'
import {AlertService} from '../../shared/service/alert.service'

@Injectable({
  providedIn: 'root'
})
export class AccountService implements OnDestroy {
  private destroy$ = new Subject<boolean>()

  constructor(
    private http: HttpClient,
    private api: ApiService,
    private appState: AppState,
    private errorService: ErrorService,
    private alertService: AlertService
  ) {
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  isLoggedIn(): boolean {
    return this.appState.isAuthenticated
  }

  checkAuthenticationStatus() {
    if (!this.appState.authenticatedUser) {
      // Try to get current default group user
      this.http.get<User>(`${environment.apiUrl}/user/current`, {withCredentials: true})
        .pipe(
          take(1),
          takeUntil(this.destroy$),
        ).subscribe(
        {
          next: user => this.appState.authenticatedUser = user,
          error: err => {
            if (err.status === 404 || err.status === 403) {
              console.warn('No default user is defined on this computer')
            } else {
              this.errorService.handleError(err)
            }
          }
        })
    }
  }

  login(id: number, password: string, success: () => void) {
    const loginForm = {id, password}
    globalLoadingWheel.show()
    this.http.post<any>(`${environment.apiUrl}/login`, loginForm, {
      withCredentials: true,
      observe: 'response' as 'body'
    })
      .pipe(
        take(1),
        takeUntil(this.destroy$)
      )
      .subscribe({
        next: (response: HttpResponse<any>) => {
          this.appState.authenticationExpiration = parseInt(response.headers.get('X-Authentication-Expiration'))
          this.appState.isAuthenticated = true
          this.setLoggedInUserFromApi()
          success()
        },
        error: err => {
          globalLoadingWheel.hide()
          if (err.status === 401 || err.status === 403) {
            this.alertService.pushError('Les identifiants entrés sont invalides')
          } else {
            this.errorService.handleError(err)
          }
        }
      })
  }

  logout(success: () => void) {
    this.api.get<void>('/logout', true).pipe(
      take(1),
      takeUntil(this.destroy$)
    )
      .subscribe({
        next: () => {
          this.appState.resetAuthenticatedUser()
          this.checkAuthenticationStatus()
          success()
        },
        error: err => this.errorService.handleError(err)
      })
  }

  hasPermission(permission: Permission): boolean {
    return this.appState.authenticatedUser && this.appState.authenticatedUser.permissions.indexOf(permission) >= 0
  }

  private setLoggedInUserFromApi() {
    this.api.get<User>('/user/current', true)
      .pipe(
        take(1),
        takeUntil(this.destroy$)
      )
      .subscribe({
        next: user => {
          this.appState.authenticatedUser = user
          // At this point the loading wheel should be visible
          globalLoadingWheel.hide()
        },
        error: err => this.errorService.handleError(err)
      })
  }
}
