import {Component} from '@angular/core'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {RecipeService} from '../../services/recipe.service'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {Validators} from '@angular/forms'
import {CompanyService} from '../../../company/service/company.service'
import {map} from 'rxjs/operators'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-add',
  templateUrl: './add.component.html',
  styleUrls: ['./add.component.sass']
})
export class AddComponent extends ErrorHandlingComponent {
  formFields: FormField[] = [
    {
      name: 'name',
      label: 'Nom',
      icon: 'form-textbox',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un nom est requis'}
      ]
    },
    {
      name: 'description',
      label: 'Description',
      icon: 'text',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une description est requise'}
      ]
    },
    {
      name: 'color',
      label: 'Couleur',
      icon: 'palette',
      type: 'color',
      defaultValue: "#ffffff",
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une couleur est requise'}
      ]
    },
    {
      name: 'gloss',
      label: 'Lustre',
      type: 'slider',
      min: 0,
      max: 100,
      defaultValue: 0,
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Le lustre de la couleur est requis'}
      ]
    },
    {
      name: 'sample',
      label: 'Échantillon',
      icon: 'pound',
      type: 'number',
      validator: Validators.min(0),
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un numéro d\'échantillon est requis'},
        {conditionFn: errors => errors.min, message: 'Le numéro d\'échantillon doit être supérieur ou égal à 0'}
      ]
    },
    {
      name: 'approbationDate',
      label: 'Date d\'approbation',
      icon: 'calendar',
      type: 'date'
    },
    {
      name: 'remark',
      label: 'Remarque',
      icon: 'text',
      type: 'text'
    },
    {
      name: 'company',
      label: 'Bannière',
      icon: 'domain',
      type: 'select',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une bannière est requise'}
      ],
      options$: this.companyService.all.pipe(map(companies => companies.map(c => {
        return {value: c.id, label: c.name}
      })))
    }
  ]

  errorHandlers = [{
    filter: error => error.type === `exists-recipe-company-name`,
    messageProducer: error => `Une couleur avec le nom ${error.name} existe déjà pour la bannière ${error.company}`
  }]

  constructor(
    private recipeService: RecipeService,
    private companyService: CompanyService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = "Nouvelle couleur"
  }

  submit(values) {
    this.subscribe(
      this.recipeService.save(values.name, values.description, values.color, values.gloss, values.sample, values.approbationDate, values.remark, values.company),
      recipe => this.urlUtils.navigateTo(`/color/edit/${recipe.id}`)
    )
  }
}
