import {Component, ViewChild} from '@angular/core'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {Recipe, recipeMixCount, RecipeStep, recipeStepCount} from '../../../shared/model/recipe.model'
import {RecipeService} from '../../services/recipe.service'
import {ActivatedRoute, Router} from '@angular/router'
import {Validators} from '@angular/forms'
import {Subject} from 'rxjs'
import {UNIT_GALLON, UNIT_LITER, UNIT_MILLILITER} from '../../../shared/units'
import {AccountService} from '../../../accounts/services/account.service'
import {EntityEditComponent} from '../../../shared/components/entity-edit/entity-edit.component'
import {ImagesEditorComponent} from '../../components/images-editor/images-editor.component'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AlertService} from '../../../shared/service/alert.service'
import {GroupService} from '../../../groups/services/group.service'
import {AppState} from '../../../shared/app-state'
import {StepTableComponent} from '../../components/step-table/step-table.component'

@Component({
  selector: 'cre-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.sass']
})
export class EditComponent extends ErrorHandlingComponent {
  readonly unitConstants = {UNIT_MILLILITER, UNIT_LITER, UNIT_GALLON}

  @ViewChild('imagesEditor') imagesEditor: ImagesEditorComponent

  recipe: Recipe | null
  groups$ = this.groupService.all
  formFields = [
    {
      name: 'name',
      label: 'Nom',
      icon: 'form-textbox',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un nom est requis'}
      ]
    },
    {
      name: 'description',
      label: 'Description',
      icon: 'text',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une description est requise'}
      ]
    },
    {
      name: 'color',
      label: 'Couleur',
      icon: 'palette',
      type: 'color',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une couleur est requise'}
      ]
    },
    {
      name: 'gloss',
      label: 'Lustre',
      type: 'slider',
      min: 0,
      max: 100,
      validator: Validators.compose([Validators.required, Validators.min(0), Validators.max(100)]),
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Le lustre de la couleur est requis'}
      ]
    },
    {
      name: 'sample',
      label: 'Échantillon',
      icon: 'pound',
      type: 'number',
      validator: Validators.min(0),
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un numéro d\'échantillon est requis'},
        {conditionFn: errors => errors.min, message: 'Le numéro d\'échantillon doit être supérieur ou égal à 0'}
      ]
    },
    {
      name: 'approbationDate',
      label: 'Date d\'approbation',
      icon: 'calendar',
      type: 'date'
    },
    {
      name: 'remark',
      label: 'Remarque',
      icon: 'text',
      type: 'text'
    },
    {
      name: 'company',
      label: 'Bannière',
      icon: 'domain',
      type: 'text',
      readonly: true,
      valueFn: recipe => recipe.company.name,
    }
  ]
  units$ = new Subject<string>()
  submittedValues: any | null

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'notfound-recipe-id',
    consumer: error => this.urlUtils.navigateTo('/color/list')
  }]

  constructor(
    private recipeService: RecipeService,
    private groupService: GroupService,
    private accountService: AccountService,
    private alertService: AlertService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribeEntityById(
      this.recipeService,
      parseInt(this.activatedRoute.snapshot.paramMap.get('id')),
      recipe => {
        this.recipe = recipe
        this.appState.title = `${recipe.name} (Modifications)`

        if (recipeMixCount(this.recipe) == 0) {
          this.alertService.pushWarning('Il n\'y a aucun mélange dans cette recette')
        }
        if (recipeStepCount(this.recipe) == 0) {
          this.alertService.pushWarning('Il n\'y a aucune étape dans cette recette')
        }
      }
    )
  }

  changeUnits(unit: string) {
    this.units$.next(unit)
  }

  submit(editComponent: EntityEditComponent, stepTable: StepTableComponent) {
    const values = editComponent.values
    this.submittedValues = values

    const steps = stepTable.mappedUpdatedSteps
    if (!this.stepsPositionsAreValid(steps)) {
      this.alertService.pushError('Les étapes ne peuvent pas avoir une position inférieure à 1')
      return
    }

    this.subscribeAndNavigate(
      this.recipeService.update(this.recipe.id, values.name, values.description, values.color, values.gloss, values.sample, values.approbationDate, values.remark, steps),
      '/color/list'
    )
  }

  delete() {
    this.subscribeAndNavigate(
      this.recipeService.delete(this.recipe.id),
      '/color/list'
    )
  }

  get loggedInUserGroupId(): number {
    return this.appState.authenticatedUser.group?.id
  }

  private stepsPositionsAreValid(steps: Map<number, RecipeStep[]>): boolean {
    let valid = true
    steps.forEach((steps, _) => {
      if (steps.find(s => s.position === 0)) {
        valid = false
        return
      }
    })
    return valid
  }
}
