import {
  AfterViewInit,
  Component,
  ContentChild,
  Directive,
  ElementRef,
  EventEmitter,
  Input,
  Output,
  ViewChild,
  ViewEncapsulation
} from '@angular/core'
import {ConfigService} from '../shared/service/config.service'
import {Config} from '../shared/model/config.model'
import {ErrorHandlingComponent, SubscribingComponent} from '../shared/components/subscribing.component'
import {ErrorService} from '../shared/service/error.service'
import {ActivatedRoute, Router} from '@angular/router'
import {formatDate, formatDateTime, getFileUrl, readFile} from '../shared/utils/utils'
import {FormControl, Validators} from '@angular/forms'
import {ConfirmBoxComponent} from '../shared/components/confirm-box/confirm-box.component'
import {environment} from '../../../environments/environment'

@Directive({
  selector: 'cre-config-label'
})
export class CreConfigLabel implements AfterViewInit {
  content: string

  constructor(
    private element: ElementRef
  ) {
  }

  ngAfterViewInit(): void {
    this.content = this.element.nativeElement.innerHTML
  }
}

@Directive({
  selector: 'cre-config-tooltip'
})
export class CreConfigTooltip implements AfterViewInit {
  content: string

  constructor(
    private element: ElementRef
  ) {
  }

  ngAfterViewInit(): void {
    this.content = this.element.nativeElement.innerHTML
  }
}

@Directive({
  selector: 'cre-config-list'
})
export class CreConfigList {
}

@Component({
  selector: 'cre-config-actions',
  templateUrl: 'config-actions.html'
})
export class CreConfigActions {
}

@Component({
  selector: 'cre-config-section',
  templateUrl: 'config-section.html'
})
export class CreConfigSection {
  @ContentChild(CreConfigActions) actions: CreConfigActions

  get hasActions(): boolean {
    return this.actions !== undefined
  }
}

@Component({
  selector: 'cre-config',
  templateUrl: 'config.html',
  styleUrls: ['config.sass']
})
export class CreConfig extends SubscribingComponent {
  @Input() config: { key: string, control: FormControl }

  @ContentChild(CreConfigLabel, {static: true}) label: CreConfigLabel
  @ContentChild(CreConfigTooltip, {static: true}) tooltip: CreConfigTooltip

  configuration: Config | null

  constructor(
    private configService: ConfigService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribe(
      this.configService.get(this.config.key),
      config => this.setConfig(config)
    )
  }

  setConfig(config: Config) {
    this.configuration = config
    this.config.control.setValue(config.content)
    if (!config.editable) {
      this.config.control.disable()
    }
  }

  get lastUpdated(): string {
    return 'Dernière mise à jour: ' + formatDateTime(this.configuration.lastUpdated)
  }
}

@Component({
  selector: 'cre-image-config',
  templateUrl: 'image.html',
  styleUrls: ['config.sass'],
  encapsulation: ViewEncapsulation.None
})
export class CreImageConfig extends CreConfig {
  @Input() previewWidth: string | null

  @Output() invalidFormat = new EventEmitter<void>()

  updatedImage: any | null

  constructor(
    configService: ConfigService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(configService, errorService, activatedRoute, router)
  }

  updateImage(file: File): any {
    readFile(file, (content) => this.updatedImage = content)
  }


  get configuredImageUrl(): string {
    return getFileUrl(this.configuration.content)
  }
}

@Component({
  selector: 'cre-bool-config',
  templateUrl: 'bool.html'
})
export class CreBoolConfig extends CreConfig {
  setConfig(config: Config) {
    super.setConfig(config)
    this.config.control.setValue(config.content === 'true')
  }
}

@Component({
  selector: 'cre-period-config',
  templateUrl: 'period.html'
})
export class CrePeriodConfig extends CreConfig {
}

@Component({
  selector: 'cre-date-config',
  templateUrl: 'date.html'
})
export class CreDateConfig extends CreConfig {
  setConfig(config: Config) {
    super.setConfig(config);
    this.config.control.setValue(formatDate(config.content))
  }
}

@Component({
  selector: 'cre-config-editor',
  templateUrl: 'editor.html'
})
export class CreConfigEditor extends ErrorHandlingComponent {
  @ViewChild('restartingConfirmBox', {static: true}) restartConfirmBox: ConfirmBoxComponent

  keys = {
    INSTANCE_NAME: Config.INSTANCE_NAME,
    INSTANCE_LOGO_PATH: Config.INSTANCE_LOGO_PATH,
    INSTANCE_ICON_PATH: Config.INSTANCE_ICON_PATH,
    INSTANCE_URL: Config.INSTANCE_URL,
    DATABASE_URL: Config.DATABASE_URL,
    DATABASE_USER: Config.DATABASE_USER,
    DATABASE_PASSWORD: Config.DATABASE_PASSWORD,
    DATABASE_VERSION: Config.DATABASE_VERSION,
    RECIPE_APPROBATION_EXPIRATION: Config.RECIPE_APPROBATION_EXPIRATION,
    TOUCH_UP_KIT_CACHE_PDF: Config.TOUCH_UP_KIT_CACHE_PDF,
    TOUCH_UP_KIT_EXPIRATION: Config.TOUCH_UP_KIT_EXPIRATION,
    BACKEND_BUILD_VERSION: Config.BACKEND_BUILD_VERSION,
    BACKEND_BUILD_TIME: Config.BACKEND_BUILD_TIME,
    JAVA_VERSION: Config.JAVA_VERSION,
    OPERATING_SYSTEM: Config.OPERATING_SYSTEM
  }
  controls = new Map<string, FormControl>()
  emergencyMode: string | null

  constructor(
    private configService: ConfigService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)

    for (let key in this.keys) {
      this.controls[this.keys[key]] = new FormControl(null, Validators.required)
    }
  }

  ngOnInit() {
    this.subscribe(
      this.configService.get(Config.EMERGENCY_MODE),
      config => {
        this.emergencyMode = config.content
      }
    )
  }

  getConfig(key: string) {
    return {key, control: this.controls[key]}
  }

  save() {
    this.subscribe(
      this.configService.set(this.controls),
      () => this.reload()
    )
  }

  restart() {
    this.subscribe(
      this.configService.restart(),
      () => this.restartConfirmBox.show()
    )
  }

  reload() {
    window.location.reload()
  }
}
